TINY = "tiny"
TINY_EN = "tiny.en"
BASE = "base"
BASE_EN = "base.en"
SMALL = "small"
SMALL_EN = "small.en"
MEDIUM = "medium"
MEDIUM_EN = "medium.en"
LARGE_V1 = "large-v1"
LARGE_V2 = "large-v2"
LARGE_V3 = "large-v3"
LARGE = "large"
